#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from zope.component import queryUtility
from zope.location import location
from zope.interface import implements
from zope.publisher.interfaces import IPublishTraverse, NotFound
from zope.security.checker import canAccess
from zope.security.interfaces import Unauthorized
from zope.security.proxy import removeSecurityProxy

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.intervention import interfaces
from schooltool.person.interfaces import IPerson
from schooltool.term.interfaces import IDateManager
from schooltool.traverser.traverser import Traverser, TraverserPlugin


class InterventionTabProxy(object):
    """Proxy used as context of the Intervention tab views."""
    implements(interfaces.IInterventionTabProxy)

    def __init__(self, user):
        self.user = user


class InterventionTabTraverserPlugin(TraverserPlugin):
    """Traverses from a user's BasicPerson object to the Intervention tab
       proxy which in turn will serve as context to the Intervention tab views.

    i.e. if you go to localhost/persons/someuser/intervention_tab this traverser
    will get called instead of standard traverser.  It returns an
    InterventionTabProxy for the user.
    """

    def traverse(self, name):
        person = IPerson(self.request.principal, None)
        if person is None or person.username != self.context.username:
            raise Unauthorized
        proxyInterventionTab = InterventionTabProxy(self.context)
        return location.LocationProxy(
            proxyInterventionTab, self.context, self.__name__)


class StudentSchoolYearsProxy(object):
    """Proxy used for traversing to a student message or goal."""
    implements(interfaces.IStudentSchoolYearsProxy)

    def __init__(self, student):
        self.student = student


class StudentSchoolYearsTraverserPlugin(TraverserPlugin):
    """Represents the first step in traversing from a student to a goal or
       message object.

    i.e. if you go to localhost/persons/somestudent/schoolyears this traverser
    will get called instead of standard traverser.  It returns the an
    StudentSchoolYearsProxy for the student.
    """

    def traverse(self, name):
        proxySchoolYears = StudentSchoolYearsProxy(self.context)
        return location.LocationProxy(
            proxySchoolYears, self.context, self.__name__)


class StudentSchoolYearProxy(object):
    """Proxy used for traversing to a student message or goal."""
    implements(interfaces.IStudentSchoolYearProxy)

    def __init__(self, year):
        self.year = removeSecurityProxy(year)


class StudentSchoolYearTraverser(Traverser):
    """Allows traversal to a student's schoolyear proxy.

    i.e. if you go to localhost/persons/somestudent/schoolyears/someyear this
    traverser will get called instead of standard traverser.  It also wraps the
    message in a location proxy so the url looks like what is above.

    The whole reason for having this traversal adaptation is to allow
    persons responsible to see messages or goals for students that they
    are not teaching, advising or administrating.
    """

    def traverse(self, name):
        app = ISchoolToolApplication(None)
        interventionRoot = app[u'schooltool.interventions']
        if name not in interventionRoot:
            raise NotFound(self.context, name, self.request)
        proxySchoolYear = StudentSchoolYearProxy(interventionRoot[name])
        return location.LocationProxy(
            proxySchoolYear, self.context, name)


class StudentMessagesGoalsProxy(object):
    """Proxy used for traversing to a student message or goal."""

    implements(interfaces.IStudentMessagesGoalsProxy)

    def __init__(self, type):
        self.type = type


class StudentMessagesGoalsTraverser(Traverser):
    """Allows traversal to a student's schoolyear proxy.

    i.e. go to localhost/persons/somestudent/schoolyears/someyear/messages or
    goals and this traverser will get called instead of standard traverser.
    It also wraps the message in a location proxy so the url looks like what
    is above.

    The whole reason for having this traversal adaptation is to allow
    persons responsible to see messages or goals for students that they
    are not teaching, advising or administrating.
    """

    def traverse(self, name):
        proxyMessagesGoals = StudentMessagesGoalsProxy(name)
        return location.LocationProxy(
            proxyMessagesGoals, self.context, name)


class StudentMessageGoalTraverser(Traverser):
    """Allows traversal to a student's schoolyear proxy.

    i.e. localhost/persons/somestudent/schoolyears/someyear/goals/somegoal or
    message and this traverser will get called instead of standard traverser.
    It also wraps the message in a location proxy so the url looks like what
    is above.

    The whole reason for having this traversal adaptation is to allow
    persons responsible to see messages or goals for students that they
    are not teaching, advising or administrating.
    """
    def traverse(self, name):
        context = removeSecurityProxy(self.context)
        yearproxy = removeSecurityProxy(context.__parent__)
        yearsproxy = removeSecurityProxy(yearproxy.__parent__)
        student = removeSecurityProxy(yearsproxy.student)
        interventionStudent = removeSecurityProxy(
            yearproxy.year[student.username])
        messagesGoals = interventionStudent[context.type]
        if name not in messagesGoals:
            raise NotFound(self.context, name, self.request)
        messageGoal = messagesGoals[name]
        if not canAccess(messageGoal, 'created'):
            raise Unauthorized
        return location.LocationProxy(
            messageGoal, self.context, name)


class InterventionStudentTraverserPlugin(TraverserPlugin):
    """Allows traversal to a student's intervention object.

    i.e. if you go to localhost/persons/somestudent/intervention this traverser
    will get called instead of standard traverser.  It returns the student's
    intervention object for the current school year.
    """

    def traverse(self, name):
        if queryUtility(IDateManager).current_term is None:
            return self.context
        interventionStudent = interfaces.IInterventionStudent(self.context)
        return interventionStudent


class SectionMessagesProxy(object):
    """Proxy used for traversing to a section's student messages."""

    implements(interfaces.ISectionMessagesProxy)

    def __init__(self, section):
        self.section = section
        self.student = None


class SectionMessagesTraverserPlugin(TraverserPlugin):
    """Allows traversal from a section object to a SectionMessagesProxy
    object, which in turn will be used to traverse to the messages
    container for a given student.

    i.e. if you go to localhost/sections/somesec/messages this traverser
    will get called instead of standard traverser.
    """

    def traverse(self, name):
        proxyMessages = SectionMessagesProxy(self.context)
        return location.LocationProxy(
            proxyMessages, self.context, self.__name__)

class SectionMessagesTraverser(Traverser):
    """Allows traversal to a student's messages container from the
    section messages proxy.

    i.e. if you go to localhost/sections/somesec/messages/someone this
    traverser will get called instead of standard traverser.  It also wraps the
    message in a location proxy so the url looks like what is above.

    The whole reason for having this traversal adaptation is to allow
    the message add view to know what section the message is for.
    """

    def traverse(self, name):
        student = ISchoolToolApplication(None)['persons'][name]
        interventionStudent = interfaces.IInterventionStudent(student)
        proxy = removeSecurityProxy(self.context)
        proxy.student = student
        messages = interventionStudent['messages']
        messages = location.LocationProxy(messages, self.context, name)
        return messages




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from zope.authentication.interfaces import IUnauthenticatedPrincipal
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.security.proxy import removeSecurityProxy

from schooltool.app.membership import URIGroup
from schooltool.contact.interfaces import IContact
from schooltool.course.interfaces import ISection
from schooltool.person.interfaces import IPerson
from schooltool.relationship.relationship import getRelatedObjects
from schooltool.securitypolicy import crowds

from schooltool.intervention import InterventionGettext as _
import interfaces


class InterventionInstructorsCrowd(crowds.Crowd):
    """Crowd of instructors of the student indicated by the given 
       intervention object."""

    title = _(u'Instructors')
    description = _(u'Instructors of a student in any of his sections.')

    def _getSections(self, ob):
        return [section for section in getRelatedObjects(ob, URIGroup)
                if ISection.providedBy(section)]

    def contains(self, principal):
        if IUnauthenticatedPrincipal.providedBy(principal):
            return False
        teacher = IPerson(principal)
        student = removeSecurityProxy(IPerson(self.context))
        for section in self._getSections(student):
            if teacher in section.instructors:
                return True
        return False


class InterventionAdvisorsCrowd(crowds.Crowd):
    """Crowd of advisors of the student indicated by the given 
       intervention object."""

    title = _(u'Advisors')
    description = _(u'Advisors of a student.')

    def contains(self, principal):
        if IUnauthenticatedPrincipal.providedBy(principal):
            return False
        teacher = IPerson(principal)
        student = removeSecurityProxy(IPerson(self.context))
        return teacher in student.advisors


class BaseResponsibleCrowd(crowds.Crowd):
    """Crowd of any user who is on the list of persons responsible for the
       message or goal."""

    def contains(self, principal):
        if IUnauthenticatedPrincipal.providedBy(principal):
            return False
        if not interfaces.IInterventionMarker.providedBy(self.context):
            return False
        responsible = interfaces.IInterventionPersonsResponsible(self.context)
        contact_id = getUtility(IIntIds).getId(IContact(IPerson(principal)))
        return contact_id in responsible


class StaffResponsibleCrowd(BaseResponsibleCrowd):
    """Crowd of school staff who are on the list of persons responsible for the
       message or goal."""

    title = _(u'Staff responsible')
    description = _(u'Staff members responsible for the message or goal.')

    def contains(self, principal):
        if not super(StaffResponsibleCrowd, self).contains(principal):
            return False
        student = removeSecurityProxy(IPerson(self.context))
        return IPerson(principal).username != student.username


class StudentResponsibleCrowd(BaseResponsibleCrowd):
    """Crowd containing the student who is in the list of persons responsible
       for the message or goal."""

    title = _(u'Students responsible')
    description = _(u'Students responsible for the message or goal.')

    def contains(self, principal):
        if not super(StudentResponsibleCrowd, self).contains(principal):
            return False
        student = removeSecurityProxy(IPerson(self.context))
        return IPerson(principal).username == student.username




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Upgrade scholtool.intervention to generation 6.

Evolution script to get rid of intervention schoolyears for which there is no
corresponding schooltool schoolyear.
"""
from zope.app.generations.utility import findObjectsProviding
from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication


def evolve(context):
    root = getRootFolder(context)

    old_site = getSite()
    apps = findObjectsProviding(root, ISchoolToolApplication)
    for app in apps:
        setSite(app)
        interventionRoot = app.get(u'schooltool.interventions', {})
        for schoolYear in interventionRoot.keys():
            if schoolYear not in app[u'schooltool.schoolyear']:
                del interventionRoot[schoolYear]

    setSite(old_site)




#!/usr/bin/env python
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation,
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
SchoolTool Intervention setup script.
"""

import os
from setuptools import setup, find_packages

if os.path.exists("version.txt"):
    version = open("version.txt").read().strip()
else:
    version = open("version.txt.in").read().strip()

def read(*rnames):
    text = open(os.path.join(os.path.dirname(__file__), *rnames)).read()
    return text

setup(
    name="schooltool.intervention",
    description="Plan and manage student interventions in SchoolTool",
    long_description=(
        read('README.txt')
        + '\n\n' +
        read('CHANGES.txt')
        ),
    version=version,
    url="http://www.schooltool.org",
    license="GPL",
    maintainer="SchoolTool Developers",
    maintainer_email="schooltool-developers@lists.launchpad.net",
    platforms=["any"],
    classifiers=["Development Status :: 4 - Beta",
    "Environment :: Web Environment",
    "Intended Audience :: End Users/Desktop",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Zope",
    "Topic :: Education"],
    package_dir={'': 'src'},
    packages=find_packages('src'),
    namespace_packages=["schooltool"],
    install_requires=['schooltool>=2.7dev',
                      'pytz',
                      'reportlab',
                      'setuptools',
                      'z3c.form',
                      'zc.catalog',
                      'zc.table',
                      'ZODB3',
                      'zope.annotation',
                      'zope.app.form',
                      'zope.app.generations',
                      'zope.authentication',
                      'zope.browser',
                      'zope.browserpage>=3.10.1',
                      'zope.catalog',
                      'zope.component',
                      'zope.container',
                      'zope.dublincore',
                      'zope.formlib>=4',
                      'zope.i18n',
                      'zope.i18nmessageid',
                      'zope.interface',
                      'zope.lifecycleevent',
                      'zope.location',
                      'zope.publisher',
                      'zope.schema',
                      'zope.security',
                      'zope.traversing'],
    extras_require={'test': ['zope.site',
                             'zope.intid',
                             'zope.app.testing',
                             'schooltool.devtools>=0.6'],
                    'gradebook': ['schooltool.gradebook'],
                   },
    include_package_data=True,
    zip_safe=False,
    entry_points="""
        [z3c.autoinclude.plugin]
        target = schooltool
        """,
    )



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
PDF Views
"""

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile

from schooltool.app.browser.report import ReportPDFView
from schooltool.person.interfaces import IPerson

from schooltool.intervention import intervention
from schooltool.intervention import InterventionGettext as _


class InterventionStudentPDFView(ReportPDFView):
    """The intervention student (PDF) view class"""

    template=ViewPageTemplateFile('rml/intervention_student_rml.pt')

    def title(self):
        person = IPerson(self.context)
        return _('Student Interventions Report: ${student}',
                 mapping={'student': '%s %s' % (person.first_name,
                                                person.last_name)})

    @property
    def goals(self):
        goals = []
        for k, v in sorted(self.context['goals'].items()):
            goals.append({
                'goal': v.goal,
                'added': v.created.strftime('%x'),
                'due': v.timeline.strftime('%x'),
                'status': v.goal_met and _('CLOSED') or _('OPEN'),
                })
        return goals

    @property
    def messages(self):
        messages = []
        for k, v in sorted(self.context['messages'].items()):
            message = v.body
            if len(message) > 40:
                message = message[:100] + '...'
            messages.append({
                'creation_date': v.created,
                'from': ', '.join(intervention.contactsName(v.sender)),
                'sent': v.created.strftime('%x'),
                'message': message,
                })
        return sorted(messages, key=lambda m: m['creation_date'], reverse=True)




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 1.

Moves dublincore settings to actual object attributes, then
remove dublincore from annotations.
"""

from zope.annotation.interfaces import IAnnotations
from zope.app.generations.utility import getRootFolder
from zope.dublincore.annotatableadapter import DCkey
from zope.dublincore.interfaces import IZopeDublinCore
from zope.security.proxy import removeSecurityProxy

from schooltool.contact.interfaces import IContact
from schooltool.intervention.interfaces import IInterventionGoal


def fixDublinCore(obj, persons):
    dublin = IZopeDublinCore(obj)
    if IInterventionGoal.providedBy(obj):
        creator = dublin.creators[0].split('.')[2]
        if creator in persons:
            obj.creator.add(IContact(persons[creator]))
    obj.created = dublin.created
    del IAnnotations(obj)[DCkey]


def evolve(context):
    """Moves dublincore settings to actual object attributes, then
       remove dublincore from annotations."""

    app = getRootFolder(context)
    interventionRoot = app.get(u'schooltool.interventions', {})
    persons = app.get(u'persons', {})
    for schoolYear in interventionRoot.values():
        for student in schoolYear.values():
            for message in student['messages'].values():
                fixDublinCore(message, persons)
            for goal in student['goals'].values():
                fixDublinCore(goal, persons)

